// name: CliqueList.C
// author: J. Michael Word
// date written: 2/7/98
// purpose: Implementation for sequence of interacting residues
//          and singletons

// **************************************************************
// NOTICE: This is free software and the source code is freely
// available. You are free to redistribute or modify under the
// conditions that (1) this notice is not removed or modified
// in any way and (2) any modified versions of the program are
// also available for free.
//               ** Absolutely no Warranty **
// Copyright (C) 1999 J. Michael Word
// **************************************************************

#ifdef OLD_STD_HDRS
#include <iostream.h>
#include <stdio.h>
#else
#include <iostream>
using std::endl;
#include <cstdio>
#endif

#include "CliqueList.h"
#include "Mover.h"

void CliqueList::describe(ostream& os) const {
   int i=0, j=0;
   os << " Singles(size " << _singles.len() << ")";
   for(Seq<MoverPtr> sgl = _singles; sgl; sgl++) {
      if (++i > 4) { i = 0; os << endl << "  "; }
      os << ":" << (*sgl)->descr();
   }
   os << endl;

   i=0;
   for(Seq< Seq<MoverPtr> > ss = _cliques; ss; ss++) {
      j = 0;
      os << " Set " << ++i << " (size " << (*ss).len() << ")";
      for(Seq<MoverPtr> c = (*ss); c; c++) {
	 if (++j > 4) { j = 0; os << endl << "  "; }
	 os << ":" << (*c)->descr();
      }
      os << endl;
   }
}

void CliqueList::sortSingletonsByDescr() {
   _singles = mpSeqSort(_singles);
}

void CliqueList::formatSingles(List<Stringclass>& cliqueNotes) const {
   char buf[200];

   for(Seq<MoverPtr> s = _singles; s; s++) {

      if ((*s)->type() == Mover::ROTATE_METHYL) {
	 ::sprintf(buf, "USER  MOD Single :%s:%s:sc=%8.3g%c  (180deg=%.3g%s)",
	    (*s)->descr().array(),(const char*)((*s)->describeOrientation()),
	    (*s)->bestScore(),(((*s)->bestHasBadBump())?'!':' '),
	    (*s)->initScore(),(((*s)->initHasBadBump())?"!":""));
      }
      else if ((*s)->canFlip()) {
	 char classcode = ' ';
	 if ((*s)->flipStateHasBadBump(0) && (*s)->flipStateHasBadBump(1)) {
	    classcode = 'C';
	 }
	 else if (abs((*s)->flipMaxScore(0) - (*s)->flipMaxScore(1)) <= 0.5) {
	    classcode = 'X';
	 }
	 else if ((*s)->flipState() != 0) {
	    classcode = 'F';
	 }
	 else {
	    classcode = 'K';
	 }
	 ::sprintf(buf, "USER  MOD Single :%s:%s:sc=%8.3g%c %c(o=%.2g%s,f=%.2g%s)",
	    (*s)->descr().array(),(const char*)((*s)->describeOrientation()),
	    (*s)->bestScore(),(((*s)->bestHasBadBump())?'!':' '), classcode,
	    (*s)->flipMaxScore(0),(((*s)->flipStateHasBadBump(0))?"!":""),
	    (*s)->flipMaxScore(1),(((*s)->flipStateHasBadBump(1))?"!":""));
      }
      else {
	 ::sprintf(buf, "USER  MOD Single :%s:%s:sc=%8.3g%c",
	    (*s)->descr().array(), (const char*)((*s)->describeOrientation()),
	    (*s)->bestScore(), (((*s)->bestHasBadBump())?'!':' '));
      }
      cliqueNotes.append(Stringclass(buf));
   }
}

void CliqueList::formatClique(List<Stringclass>& cliqueNotes, int c) const {
   Seq<MoverPtr> s;
   char buf[200];
   int i = 0;

   if (c >= 0 && _cliques.len() > c) {
      i = 0;
      for(Seq< Seq<MoverPtr> > ss = _cliques; ss; ss++) {
	 if (i++ == c) { s = *ss; break; }
      }
   }
   if (s.len() > 0) {
      i = 0;
      for(; s; s++) {

	 if ((*s)->type() == Mover::ROTATE_METHYL) {
	    ::sprintf(buf, "USER  MOD Set%2d.%d:%s:%s:sc=%8.3g%c  (180deg=%.3g%s)",
	       c+1, i+1,
	       (*s)->descr().array(),(const char*)((*s)->describeOrientation()),
	       (*s)->bestScore(),(((*s)->bestHasBadBump())?'!':' '),
	       (*s)->initScore(),(((*s)->initHasBadBump())?"!":""));
	 }
	 else if ((*s)->canFlip()) {
	    char classcode = ' ';
	    if ((*s)->flipStateHasBadBump(0) && (*s)->flipStateHasBadBump(1)) {
	       classcode = 'C';
	    }
	    else if (abs((*s)->flipMaxScore(0) - (*s)->flipMaxScore(1)) <= 0.5) {
	       classcode = 'X';
	    }
	    else if ((*s)->flipState() != 0) {
	       classcode = 'F';
	    }
	    else {
	       classcode = 'K';
	    }
	    ::sprintf(buf, "USER  MOD Set%2d.%d:%s:%s:sc=%8.3g%c %c(o=%.2g%s,f=%.2g%s)",
		  c+1, i+1,
		  (*s)->descr().array(),(const char*)((*s)->describeOrientation()),
		  (*s)->bestScore(),(((*s)->bestHasBadBump())?'!':' '), classcode,
		  (*s)->flipMaxScore(0),(((*s)->flipStateHasBadBump(0))?"!":""),
		  (*s)->flipMaxScore(1),(((*s)->flipStateHasBadBump(1))?"!":""));
	 }
	 else {
	    ::sprintf(buf, "USER  MOD Set%2d.%d:%s:%s:sc=%8.3g%c",
		  c+1, i+1,
		  (*s)->descr().array(), (const char*)((*s)->describeOrientation()),
		  (*s)->bestScore(), (((*s)->bestHasBadBump())?'!':' '));
	 }
	 cliqueNotes.append(Stringclass(buf));
	 i++;
      }
   }
}
